# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Pronouns Data Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection


class Pronouns:
    """Pronouns information retrieval class.

    Contains methods used to retrieve available pronouns.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def retrieve_all(self) -> list[dict[int, str]]:
        """Retrieves all pronouns.

        :return: A list of dictionaries containing pronouns IDs and
            corresponding pronouns
        """
        query = """
            SELECT pronounsid, pronouns
            FROM ww_pronouns
            ORDER BY pronounsid ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        pronouns = []
        for row in results:
            pronouns.append(
                {
                    "id": row["pronounsid"],
                    "pronouns": row["pronouns"],
                }
            )

        return pronouns

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all pronouns IDs, sorted by ID.

        :return: A list of pronouns IDs as integers
        """
        cursor = self.database_connection.cursor(dictionary=False)
        query = """
            SELECT pronounsid
            FROM ww_pronouns
            ORDER BY pronounsid ASC;
        """
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [row[0] for row in results]

    def retrieve_all_as_dict(self) -> dict[int, str]:
        """Retrieves all pronouns as a dictionary.

        :return: A dictionary of pronouns IDs and pronoun strings,
            ordered by pronouns ID
        """
        query = """
            SELECT pronounsid, pronouns
            FROM ww_pronouns
            ORDER BY pronounsid ASC;
        """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        return {row["pronounsid"]: row["pronouns"] for row in results}

    def retrieve_all_pronouns(self) -> list[str]:
        """Retrieves all pronouns names.

        :return: A list of pronouns names, ordered by pronouns ID
        """
        query = """
            SELECT pronouns
            FROM ww_pronouns
            ORDER BY pronounsid ASC;
        """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [row[0] for row in results]

    def retrieve_by_id(self, pronouns_id: int) -> dict[int, str]:
        """Retrieves pronouns information.

        :param pronouns_id: Pronouns ID
        :return: A dictionary containing pronouns ID and corresponding
            pronouns
        """
        query = """
            SELECT pronounsid, pronouns
            FROM ww_pronouns
            WHERE pronounsid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (pronouns_id,))
        result = cursor.fetchone()

        if not result:
            return {}

        return {
            "id": result["pronounsid"],
            "pronouns": result["pronouns"],
        }



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats: Scorekeeper module."""

from wwdtm.scorekeeper.appearances import ScorekeeperAppearances
from wwdtm.scorekeeper.scorekeeper import Scorekeeper
from wwdtm.scorekeeper.utility import ScorekeeperUtility



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
# pylint: disable=C0206
"""Wait Wait Stats Show Information Retrieval Functions."""

import datetime
from decimal import Decimal
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.show.info import ShowInfo
from wwdtm.show.info_multiple import ShowInfoMultiple
from wwdtm.show.utility import ShowUtility
from wwdtm.validation import valid_int_id


class Show:
    """Show retrieval class.

    Contains methods used to retrieve basic and detailed show
    information, including hosts, scorekeepers, guests, panelists and
    scores.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.info = ShowInfo(database_connection=self.database_connection)
        self.info_multiple = ShowInfoMultiple(
            database_connection=self.database_connection
        )
        self.utility = ShowUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, Any]]:
        """Retrieves basic show information for all shows.

        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        query = """
            SELECT showid AS id, showdate AS date,
            bestof AS best_of, repeatshowid AS repeat_show_id,
            showurl AS show_url
            FROM ww_shows
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = []
        for row in results:
            show = {
                "id": row["id"],
                "date": row["date"].isoformat(),
                "best_of": bool(row["best_of"]),
                "repeat_show": bool(row["repeat_show_id"]),
                "show_url": row["show_url"],
            }

            if row["repeat_show_id"]:
                show["original_show_id"] = row["repeat_show_id"]
                show["original_show_date"] = self.utility.convert_id_to_date(
                    row["repeat_show_id"]
                )

            shows.append(show)

        return shows

    def retrieve_all_best_ofs(self, inclusive: bool = True) -> list[dict[str, Any]]:
        """Retrieves basic show information for all Best Of shows.

        The list of Best Of shows includes repeat Best Of shows.

        :param inclusive: Include repeat shows in the list of Best Of
            shows
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        if inclusive:
            query = """
                SELECT showid AS id, showdate AS date,
                bestof AS best_of, repeatshowid AS repeat_show_id,
                showurl AS show_url
                FROM ww_shows
                WHERE bestof = 1
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid AS id, showdate AS date,
                bestof AS best_of, repeatshowid AS repeat_show_id,
                showurl AS show_url
                FROM ww_shows
                WHERE bestof = 1 AND repeatshowid IS NULL
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = []
        for row in results:
            show = {
                "id": row["id"],
                "date": row["date"].isoformat(),
                "best_of": bool(row["best_of"]),
                "repeat_show": bool(row["repeat_show_id"]),
                "show_url": row["show_url"],
            }

            if row["repeat_show_id"]:
                show["original_show_id"] = row["repeat_show_id"]
                show["original_show_date"] = self.utility.convert_id_to_date(
                    row["repeat_show_id"]
                )

            shows.append(show)

        return shows

    def retrieve_all_best_ofs_details(
        self, inclusive: bool = True, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for all Best Of shows.

        The list of Best Of shows includes repeat Best Of shows.

        :param inclusive: Include repeat shows in the list of Best Of
            shows
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        if inclusive:
            query = """
                SELECT showid
                FROM ww_shows
                WHERE bestof = 1
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid
                FROM ww_shows
                WHERE bestof = 1 AND repeatshowid IS NULL
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_all_repeat_best_ofs(self) -> list[dict[str, Any]]:
        """Retrieves basic show information for all Repeat Best Of shows.

        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        query = """
            SELECT showid AS id, showdate AS date,
            bestof AS best_of, repeatshowid AS repeat_show_id,
            showurl AS show_url
            FROM ww_shows
            WHERE bestof = 1 AND repeatshowid IS NOT NULL
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = []
        for row in results:
            show = {
                "id": row["id"],
                "date": row["date"].isoformat(),
                "best_of": bool(row["best_of"]),
                "repeat_show": bool(row["repeat_show_id"]),
                "show_url": row["show_url"],
            }

            if row["repeat_show_id"]:
                show["original_show_id"] = row["repeat_show_id"]
                show["original_show_date"] = self.utility.convert_id_to_date(
                    row["repeat_show_id"]
                )

            shows.append(show)

        return shows

    def retrieve_all_best_of_repeats(self) -> list[dict[str, Any]]:
        """Alias for :py:meth:`wwdtm.show.Show.retrieve_all_repeat_best_ofs`.

        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        return self.retrieve_all_repeat_best_ofs()

    def retrieve_all_repeat_best_ofs_details(
        self, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for all Repeat Best Of shows.

        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        query = """
            SELECT showid
            FROM ww_shows
            WHERE bestof = 1 AND repeatshowid IS NOT NULL
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_all_best_of_repeats_details(
        self, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Alias for :py:meth:`wwdtm.show.Show.retrieve_all_repeat_best_ofs_details`.

        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        return self.retrieve_all_repeat_best_ofs_details(
            include_decimal_scores=include_decimal_scores
        )

    def retrieve_all_repeats(self, inclusive: bool = True) -> list[dict[str, Any]]:
        """Retrieves basic show information for all repeat shows.

        :param inclusive: Include Best Of shows in repeat show counts
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        if inclusive:
            query = """
                SELECT showid AS id, showdate AS date,
                bestof AS best_of, repeatshowid AS repeat_show_id,
                showurl AS show_url
                FROM ww_shows
                WHERE repeatshowid IS NOT NULL
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid AS id, showdate AS date,
                bestof AS best_of, repeatshowid AS repeat_show_id,
                showurl AS show_url
                FROM ww_shows
                WHERE repeatshowid IS NOT NULL AND bestof = 0
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = []
        for row in results:
            show = {
                "id": row["id"],
                "date": row["date"].isoformat(),
                "best_of": bool(row["best_of"]),
                "repeat_show": bool(row["repeat_show_id"]),
                "show_url": row["show_url"],
            }

            if row["repeat_show_id"]:
                show["original_show_id"] = row["repeat_show_id"]
                show["original_show_date"] = self.utility.convert_id_to_date(
                    row["repeat_show_id"]
                )

            shows.append(show)

        return shows

    def retrieve_all_repeats_details(
        self, inclusive: bool = True, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for all repeat shows.

        :param inclusive: Include Best Of shows in repeat show counts
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        if inclusive:
            query = """
                SELECT showid
                FROM ww_shows
                WHERE repeatshowid IS NOT NULL
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid
                FROM ww_shows
                WHERE repeatshowid IS NOT NULL AND bestof = 0
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_all_details(
        self, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Returns a list of dictionaries with show information and details for all shows.

        :param include_decimal_scores: Flag set to include panelist decimal
            scores, if available
        :return: List of all shows and their corresponding details.
            If show information could not be retrieved, an empty list
            will be returned.
        """
        info = self.info_multiple.retrieve_core_info_all()

        if not info:
            return []

        panelists = self.info_multiple.retrieve_panelist_info_all(
            include_decimal_scores=include_decimal_scores
        )
        bluffs = self.info_multiple.retrieve_bluff_info_all()
        guests = self.info_multiple.retrieve_guest_info_all()

        shows = []
        for show in info:
            info[show]["panelists"] = panelists.get(info[show]["id"], [])
            info[show]["bluffs"] = bluffs.get(info[show]["id"], {})
            info[show]["guests"] = guests.get(info[show]["id"], [])
            shows.append(info[show])

        return shows

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all show IDs, sorted by show date.

        :return: A list of all show IDs as integers
        """
        query = """
            SELECT showid FROM ww_shows ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_dates(self) -> list[str]:
        """Retrieves all show dates, sorted by show date.

        :return: A list of all show date strings in ``YYYY-MM-DD``
            format
        """
        query = "SELECT showdate FROM ww_shows ORDER BY showdate ASC;"
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0].isoformat() for v in results]

    def retrieve_all_dates_tuple(self) -> list[tuple[int, int, int]]:
        """Retrieves all show dates as a tuple.

        :return: A list of all show dates as a tuple of year, month and
            day
        """
        query = """
            SELECT YEAR(showdate), MONTH(showdate), DAY(showdate)
            FROM ww_shows
            ORDER BY YEAR(showdate) ASC, MONTH(showdate) ASC,
            DAY(showdate) ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [tuple(v) for v in results]

    def retrieve_all_show_years_months(self) -> list[str]:
        """Retrieves all show years and months.

        :return: A list of all show years and month as a string in
            ``YYYY-MM`` format
        """
        query = """
            SELECT DISTINCT YEAR(showdate), MONTH(showdate)
            FROM ww_shows
            ORDER BY YEAR(showdate) ASC, MONTH(showdate) ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [f"{v[0]}-{v[1]}" for v in results]

    def retrieve_all_shows_years_months_tuple(self) -> list[tuple[int, int]]:
        """Retrieves all show years and months as a tuple.

        :return: A list of all show dates as a tuple of year and month
        """
        query = """
            SELECT DISTINCT YEAR(showdate), MONTH(showdate)
            FROM ww_shows
            ORDER BY YEAR(showdate) ASC, MONTH(showdate) ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [tuple(v) for v in results]

    def retrieve_best_ofs_by_year(
        self, year: int, inclusive: bool = True
    ) -> list[dict[str, Any]]:
        """Retrieves basic show information for Best Of shows by year.

        The list of Best Of shows includes repeat Best Of shows.

        :param year: Four-digit year
        :param inclusive: Include repeat shows in the list of Best Of
            shows
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        if inclusive:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND bestof = 1
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND bestof = 1
                AND repeatshowid IS NULL
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return None

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_best_ofs_details_by_year(
        self,
        year: int,
        inclusive: bool = True,
        include_decimal_scores: bool = False,
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for Best Of shows by year.

        :param year: Four-digit year
        :param inclusive: Include repeat shows in the list of Best Of
            shows
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: List of recent shows and corresponding details. If show
            information could not be retrieved, an empty list will be
            returned.
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        if inclusive:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND bestof = 1
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND bestof = 1
                AND repeatshowid IS NULL
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                show, include_decimal_scores=include_decimal_scores
            )
            info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(show)
            info[show]["guests"] = self.info.retrieve_guest_info_by_id(show)
            shows.append(info[show])

        return shows

    def retrieve_by_date(self, year: int, month: int, day: int) -> dict[str, Any]:
        """Retrieves basic show information.

        :param year: Four-digit year
        :param month: One or two-digit month
        :param day: One or two-digit day
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable) and show URL at
            NPR.org
        """
        id_ = self.utility.convert_date_to_id(year, month, day)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_by_date_string(self, date_string: str) -> dict[str, Any]:
        """Retrieves basic show information.

        :param date_string: Show date in ``YYYY-MM-DD`` format
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable) and show URL at
            NPR.org
        """
        try:
            parsed_date = datetime.datetime.strptime(date_string, "%Y-%m-%d")
        except ValueError:
            return {}

        id_ = self.utility.convert_date_to_id(
            parsed_date.year, parsed_date.month, parsed_date.day
        )

        return self.retrieve_by_id(id_)

    def retrieve_by_id(self, show_id: int) -> dict[str, Any]:
        """Retrieves basic show information.

        :param show_id: Show ID
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable) and show URL at
            NPR.org
        """
        if not valid_int_id(show_id):
            return {}

        query = """
            SELECT showid AS id, showdate AS date,
            bestof AS best_of, repeatshowid AS repeat_show_id,
            showurl AS show_url
            FROM ww_shows
            WHERE showid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        info = {
            "id": result["id"],
            "date": result["date"].isoformat(),
            "best_of": bool(result["best_of"]),
            "repeat_show": bool(result["repeat_show_id"]),
            "show_url": result["show_url"],
        }

        if result["repeat_show_id"]:
            info["original_show_id"] = result["repeat_show_id"]
            info["original_show_date"] = self.utility.convert_id_to_date(
                result["repeat_show_id"]
            )

        return info

    def retrieve_by_month_day(self, month: int, day: int) -> list[dict[str, Any]]:
        """Retrieves basic show information for shows by month and day.

        :param month: One or two-digit month
        :param day: One or two-digit day
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        if not 1 <= month <= 12 or not 1 <= day <= 31:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE MONTH(showdate) = %s AND DAY(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                month,
                day,
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_by_year(self, year: int) -> list[dict[str, Any]]:
        """Retrieves basic show information by year.

        :param year: Four-digit year
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_by_year_month(self, year: int, month: int) -> list[dict[str, Any]]:
        """Retrieves basic show information by year and month.

        :param year: Four-digit year
        :param month: One or two-digit month
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        try:
            parsed_year_month = datetime.datetime.strptime(
                f"{year:04d}-{month:02d}-01", "%Y-%m-%d"
            )
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s AND MONTH(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                parsed_year_month.year,
                parsed_year_month.month,
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_counts_by_year(
        self, year: int, inclusive: bool = False
    ) -> dict[str, int]:
        """Retrieves show counts by year.

        Counts of Best Of shows includes repeat Best Of shows.

        :param year: Four-digit year
        :param inclusive: Include Best Of shows in repeat show counts
        :return: A dictionary containing counts for all shows, Best Of
            shows, repeat shows, and repeat Best Of shows
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return {}

        if inclusive:
            query = """
                SELECT
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 0 AND repeatshowid IS NULL) AS 'regular',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 1) AS 'bestof',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND repeatshowid IS NOT NULL) AS 'repeat',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 1 AND repeatshowid IS NOT NULL) AS 'repeat_bestof';
            """
        else:
            query = """
                SELECT
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 0 AND repeatshowid IS NULL) AS 'regular',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 1) AS 'bestof',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 0 AND repeatshowid IS NOT NULL) AS 'repeat',
                (SELECT COUNT(showid) FROM ww_shows
                    WHERE YEAR(showdate) = %s AND showdate <= NOW()
                    AND bestof = 1 AND repeatshowid IS NOT NULL) AS 'repeat_bestof';
            """

        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(
            query,
            (
                parsed_year.year,
                parsed_year.year,
                parsed_year.year,
                parsed_year.year,
            ),
        )
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "regular": result["regular"],
            "best_of": result["bestof"],
            "repeat": result["repeat"],
            "repeat_best_of": result["repeat_bestof"],
            "total": (
                result["regular"]
                + result["bestof"]
                + result["repeat"]
                + result["repeat_bestof"]
            ),
        }

    def retrieve_all_counts_by_year(
        self, inclusive: bool = False
    ) -> dict[int, dict[str, int]]:
        """Retrieves show counts for all years, grouped by year.

        :return: A dictionary with year as keys with corresponding
            counts for all shows, Best Of shows, repeat shows, and
            repeat Best Of shows as values
        """
        years = self.retrieve_years()
        if not years:
            return {}

        all_counts = {}
        for year in years:
            year_counts = self.retrieve_counts_by_year(year=year, inclusive=inclusive)

            if year_counts:
                all_counts[year] = year_counts
            else:
                all_counts[year] = {}

        return all_counts

    def retrieve_details_by_date(
        self, year: int, month: int, day: int, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves detailed show information.

        :param year: Four-digit year
        :param month: One or two-digit month
        :param day: One or two digit day
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable), show URL at
            NPR.org, host, scorekeeper, location, panelists and guests
        """
        id_ = self.utility.convert_date_to_id(year, month, day)
        if not id_:
            return {}

        return self.retrieve_details_by_id(
            id_, include_decimal_scores=include_decimal_scores
        )

    def retrieve_details_by_date_string(
        self, date_string: str, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves detailed show information.

        :param date_string: Show date in ``YYYY-MM-DD`` format
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable), show URL at
            NPR.org, host, scorekeeper, location, panelists and guests
        """
        try:
            parsed_date = datetime.datetime.strptime(date_string, "%Y-%m-%d")
        except ValueError:
            return {}

        id_ = self.utility.convert_date_to_id(
            parsed_date.year, parsed_date.month, parsed_date.day
        )

        return self.retrieve_details_by_id(
            id_, include_decimal_scores=include_decimal_scores
        )

    def retrieve_details_by_id(
        self, show_id: int, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieve detailed show information.

        :param show_id: Show ID
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable), show URL at
            NPR.org, host, scorekeeper, location, panelists and guests
        """
        if not valid_int_id(show_id):
            return {}

        info = self.info.retrieve_core_info_by_id(show_id)
        if not info:
            return {}

        info["panelists"] = self.info.retrieve_panelist_info_by_id(
            show_id, include_decimal_scores=include_decimal_scores
        )
        info["bluffs"] = self.info.retrieve_bluff_info_by_id(show_id)
        info["guests"] = self.info.retrieve_guest_info_by_id(show_id)

        return info

    def retrieve_details_by_month_day(
        self, month: int, day: int, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information by month and day.

        :param month: One or two-digit month
        :param day: One or two-digit day
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        if not 1 <= month <= 12 or not 1 <= day <= 31:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE MONTH(showdate) = %s AND DAY(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                month,
                day,
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_details_by_year(
        self, year: int, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information by year.

        :param year: Four-digit year
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_details_by_year_month(
        self, year: int, month: int, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information by year and month.

        :param year: Four-digit year
        :param month: One or two-digit month
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable), show URL
            at NPR.org, host, scorekeeper, location, panelists and
            guests
        """
        try:
            parsed_year_month = datetime.datetime.strptime(
                f"{year:04d}-{month:02d}-01", "%Y-%m-%d"
            )
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s AND MONTH(showdate) = %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                parsed_year_month.year,
                parsed_year_month.month,
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            if info[show]:
                info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                    info[show]["id"], include_decimal_scores=include_decimal_scores
                )
                info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(
                    info[show]["id"]
                )
                info[show]["guests"] = self.info.retrieve_guest_info_by_id(
                    info[show]["id"]
                )
                shows.append(info[show])

        return shows

    def retrieve_months_by_year(self, year: int) -> list[int]:
        """Retrieves show months for a year.

        :param year: Four-digit year
        :return: A list of available show months
        """
        try:
            _ = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        query = """
            SELECT DISTINCT MONTH(showdate)
            FROM ww_shows
            WHERE YEAR(showdate) = %s
            ORDER BY MONTH(showdate) ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_random_id(self) -> int:
        """Retrieves an ID for a random show.

        :return: ID for a random show.
        """
        query = """
            SELECT showid FROM ww_shows
            WHERE showdate <= NOW()
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_id_by_year(self, year: int) -> int:
        """Retrieves an ID for a random show for a specific year.

        :return: ID for a random show.
        """
        query = """
            SELECT showid FROM ww_shows
            WHERE showdate <= NOW()
            AND YEAR(showdate) = %s
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (year,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_date(self) -> str:
        """Retrieves a date for a random show.

        :return: show date string for a random show, in YYYY-MM-DD format.
        """
        query = """
            SELECT showdate FROM ww_shows
            WHERE showdate <= NOW()
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0].isoformat()

    def retrieve_random_date_object(self) -> datetime.date:
        """Retrieves a date object for a random show.

        :return: Date object for a random show.
        """
        query = """
            SELECT showdate FROM ww_shows
            WHERE showdate <= NOW()
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_date_by_year(self, year: int) -> str:
        """Retrieves a date for a random show for a specific year.

        :return: show date string for a random show, in YYYY-MM-DD format.
        """
        query = """
            SELECT showdate FROM ww_shows
            WHERE showdate <= NOW()
            AND YEAR(showdate) = %s
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (year,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0].isoformat()

    def retrieve_random_date_object_by_year(self, year: int) -> datetime.date:
        """Retrieves a date object for a random show for a specific year.

        :return: Date object for a random show.
        """
        query = """
            SELECT showdate FROM ww_shows
            WHERE showdate <= NOW()
            AND YEAR(showdate) = %s
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (year,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random(self) -> dict[str, Any]:
        """Retrieves information for a random show.

        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable) and show URL at
            NPR.org
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_by_id(show_id=_id)

    def retrieve_random_by_year(self, year: int) -> dict[str, Any]:
        """Retrieves information for a random show for a specific year.

        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable) and show URL at
            NPR.org
        """
        _id = self.retrieve_random_id_by_year(year=year)

        if not _id:
            return None

        return self.retrieve_by_id(show_id=_id)

    def retrieve_random_details(
        self, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves information and appearances for a random show.

        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable), show URL at
            NPR.org, host, scorekeeper, location, panelists and guests
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_details_by_id(
            show_id=_id, include_decimal_scores=include_decimal_scores
        )

    def retrieve_random_details_by_year(
        self, year: int, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves information and appearances for a random show for a specific year.

        :return: A dictionary containing show ID, show date, Best Of
            show flag, repeat show ID (if applicable), show URL at
            NPR.org, host, scorekeeper, location, panelists and guests
        """
        _id = self.retrieve_random_id_by_year(year=year)

        if not _id:
            return None

        return self.retrieve_details_by_id(
            show_id=_id, include_decimal_scores=include_decimal_scores
        )

    def retrieve_recent(
        self,
        include_days_ahead: int = 7,
        include_days_back: int = 32,
    ) -> list[dict[str, Any]]:
        """Retrieves basic show information for recent shows.

        :param include_days_ahead: Number of days in the future to
            include
        :param include_days_back: Number of days in the past to
            include
        :return: List of recent shows and corresponding information. If
            show information could not be retrieved, an empty list will
            be returned.
        """
        try:
            past_days = int(include_days_back)
            future_days = int(include_days_ahead)
        except ValueError:
            return []

        try:
            past_date = datetime.datetime.now() - datetime.timedelta(days=past_days)
            future_date = datetime.datetime.now() + datetime.timedelta(days=future_days)
        except OverflowError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE showdate >= %s AND showdate <= %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                past_date.isoformat(),
                future_date.isoformat(),
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_recent_details(
        self,
        include_days_ahead: int = 7,
        include_days_back: int = 32,
        include_decimal_scores: bool = False,
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for recent shows.

        :param include_days_ahead: Number of days in the future to
            include
        :param include_days_back: Number of days in the past to
            include
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: List of recent shows and corresponding details. If show
            information could not be retrieved, an empty list will be
            returned.
        """
        try:
            past_days = int(include_days_back)
            future_days = int(include_days_ahead)
        except ValueError:
            return []

        try:
            past_date = datetime.datetime.now() - datetime.timedelta(days=past_days)
            future_date = datetime.datetime.now() + datetime.timedelta(days=future_days)
        except OverflowError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE showdate >= %s AND showdate <= %s
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(
            query,
            (
                past_date.isoformat(),
                future_date.isoformat(),
            ),
        )
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                show, include_decimal_scores=include_decimal_scores
            )
            info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(show)
            info[show]["guests"] = self.info.retrieve_guest_info_by_id(show)
            shows.append(info[show])

        return shows

    def retrieve_repeat_best_ofs_by_year(self, year: int) -> list[dict[str, Any]]:
        """Retrieves basic show information for repeat Best Of shows by year.

        :param year: Four-digit year
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s AND bestof = 1
            AND repeatshowid IS NOT NULL
            ORDER BY showdate ASC;
        """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return None

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_repeat_best_ofs_details_by_year(
        self,
        year: int,
        include_decimal_scores: bool = False,
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for repeat Best Of shows by year.

        :param year: Four-digit year
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: List of recent shows and corresponding details. If show
            information could not be retrieved, an empty list will be
            returned.
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s AND bestof = 1
            AND repeatshowid IS NOT NULL
            ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                show, include_decimal_scores=include_decimal_scores
            )
            info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(show)
            info[show]["guests"] = self.info.retrieve_guest_info_by_id(show)
            shows.append(info[show])

        return shows

    def retrieve_repeats_by_year(
        self, year: int, inclusive: bool = True
    ) -> list[dict[str, Any]]:
        """Retrieves basic show information for repeat shows by year.

        The list of repeat Best Of shows includes repeat Best Of shows.

        :param year: Four-digit year
        :param inclusive: Include Best Of shows in the list of repeat
            shows
        :return: A list of dictionaries containing show ID, show date,
            Best Of show flag, repeat show ID (if applicable) and show
            URL at NPR.org
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        if inclusive:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND repeatshowid IS NOT NULL
                ORDER BY showdate ASC;
            """
        else:
            query = """
            SELECT showid FROM ww_shows
            WHERE YEAR(showdate) = %s AND repeatshowid IS NOT NULL
            AND bestof = 0
            ORDER BY showdate ASC;
        """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return None

        return [self.retrieve_by_id(v[0]) for v in results]

    def retrieve_repeats_details_by_year(
        self,
        year: int,
        inclusive: bool = True,
        include_decimal_scores: bool = False,
    ) -> list[dict[str, Any]]:
        """Retrieves detailed show information for repeat shows by year.

        The list of repeat Best Of shows includes repeat Best Of shows.

        :param year: Four-digit year
        :param inclusive: Include Best Of shows in the list of repeat
            shows
        :param include_decimal_scores: A boolean to determine if decimal
            scores should be included
        :return: List of recent shows and corresponding details. If show
            information could not be retrieved, an empty list will be
            returned.
        """
        try:
            parsed_year = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        if inclusive:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND repeatshowid IS NOT NULL
                ORDER BY showdate ASC;
            """
        else:
            query = """
                SELECT showid FROM ww_shows
                WHERE YEAR(showdate) = %s AND repeatshowid IS NOT NULL
                AND bestof = 0
                ORDER BY showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (parsed_year.year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        show_ids = [v[0] for v in results]
        info = self.info_multiple.retrieve_core_info_by_ids(show_ids)

        if not info:
            return []

        shows = []
        for show in info:
            info[show]["panelists"] = self.info.retrieve_panelist_info_by_id(
                show, include_decimal_scores=include_decimal_scores
            )
            info[show]["bluffs"] = self.info.retrieve_bluff_info_by_id(show)
            info[show]["guests"] = self.info.retrieve_guest_info_by_id(show)
            shows.append(info[show])

        return shows

    def retrieve_scores_by_year(
        self, year: int, use_decimal_scores: bool = False
    ) -> list[tuple[str, int | Decimal]]:
        """Retrieves panelist scores for all shows as a tuple.

        :param year: Four-digit year
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A list of tuples each containing show date and panelist
            scores
        """
        try:
            _ = datetime.datetime.strptime(f"{year:04d}", "%Y")
        except ValueError:
            return []

        if use_decimal_scores:
            query = """
                SELECT s.showdate AS date, pm.panelistscore_decimal AS score
                FROM ww_showpnlmap pm
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE s.bestof = 0 AND s.repeatshowid IS NULL
                AND pm.panelistscore_decimal IS NOT NULL
                AND YEAR(s.showdate) = %s
                ORDER BY s.showdate ASC, pm.panelistscore_decimal ASC;
                """
        else:
            query = """
                SELECT s.showdate AS date, pm.panelistscore AS score
                FROM ww_showpnlmap pm
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE s.bestof = 0 AND s.repeatshowid IS NULL
                AND pm.panelistscore IS NOT NULL
                AND YEAR(s.showdate) = %s
                ORDER BY s.showdate ASC, pm.panelistscore ASC;
                """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (year,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = {}
        for row in results:
            date = row["date"].isoformat()
            if date not in shows:
                shows[date] = []

            shows[date].append(row["score"])

        show_scores = []
        for show in shows:
            show_score = shows[show]
            show_score.insert(0, show)
            show_scores.append(tuple(show_score))

        return show_scores

    def retrieve_years(self) -> list[int]:
        """Retrieves show years.

        :return: A list of available show years
        """
        query = """
            SELECT DISTINCT YEAR(showdate)
            FROM ww_shows
            ORDER BY YEAR(showdate) ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object :py:class:`wwdtm.show.Show`."""

import datetime
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.show import Show


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_show_retrieve_all():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("inclusive", [True, False])
def test_show_retrieve_all_best_ofs(inclusive: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_best_ofs`.

    :param inclusive: Include Repeat shows in the list of Best Of
        shows
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_best_ofs(inclusive=inclusive)

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize(
    "inclusive, include_decimal_scores",
    [(True, True), (True, False), (False, True), (False, False)],
)
def test_show_retrieve_all_best_ofs_details(
    inclusive: bool, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_best_ofs_details`.

    :param inclusive: Include Repeat shows in the list of Best Of
        shows
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_best_ofs_details(
        inclusive=inclusive, include_decimal_scores=include_decimal_scores
    )

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


def test_show_retrieve_all_best_of_repeats():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_best_of_repeats`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_best_of_repeats()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_all_best_of_repeats_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_best_of_repeats_details`.

    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_best_of_repeats_details(
        include_decimal_scores=include_decimal_scores
    )

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_all_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_details`.

    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_details(include_decimal_scores=include_decimal_scores)

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


def test_show_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_ids`."""
    show = Show(connect_dict=get_connect_dict())
    ids = show.retrieve_all_ids()

    assert ids, "No show IDs could be retrieved"


def test_show_retrieve_all_dates():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_dates`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_dates()

    assert dates, "No show dates could be retrieved"


def test_show_retrieve_all_dates_tuple():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_dates_tuple`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_dates_tuple()

    assert dates, "No show dates could be retrieved"
    assert isinstance(dates[0], tuple), "First list item is not a tuple"


@pytest.mark.parametrize("inclusive", [True, False])
def test_show_retrieve_all_repeats(inclusive: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_repeats`.

    :param inclusive: Include Best Of shows in the list of repeat
        shows
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_repeats(inclusive=inclusive)

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize(
    "inclusive, include_decimal_scores",
    [(True, True), (True, False), (False, True), (False, False)],
)
def test_show_retrieve_all_repeat_details(
    inclusive: bool, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_repeat_details`.

    :param inclusive: Include Best Of shows in the list of repeat
        shows
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_repeats_details(
        inclusive=inclusive, include_decimal_scores=include_decimal_scores
    )

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


def test_show_retrieve_all_repeat_best_ofs():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_repeat_best_ofs`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_repeat_best_ofs()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_all_repeat_best_ofs_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_repeat_best_ofs_details`.

    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_repeat_best_ofs_details(
        include_decimal_scores=include_decimal_scores
    )

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


def test_show_retrieve_all_show_years_months():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_show_years_months`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_show_years_months()

    assert dates, "No dates could be retrieved"
    assert isinstance(dates[0], str), "First list item is not a string"


def test_show_retrieve_all_show_years_months_tuple():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_shows_years_months_tuple`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_shows_years_months_tuple()

    assert dates, "No dates could be retrieved"
    assert isinstance(dates[0], tuple), "First list item is not a tuple"


@pytest.mark.parametrize(
    "year, inclusive", [(1998, True), (1998, False), (2008, True), (2008, False)]
)
def test_show_retrieve_best_ofs_by_year(year: int, inclusive: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_best_ofs_by_year`.

    :param year: Four digit year to test retrieving show information
    :param inclusive: Include Repeat shows in the list of Best Of
        shows
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_best_ofs_by_year(year=year, inclusive=inclusive)

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "best_of" in shows[0], "'best_of' was not returned for the first list item"
    assert shows[0]["best_of"], "'best_of' value for the first list item is not valid"


@pytest.mark.parametrize(
    "year, inclusive, include_decimal_scores",
    [
        (1998, True, True),
        (1998, True, False),
        (1998, False, True),
        (1998, False, False),
        (2008, True, True),
        (2008, True, False),
        (2008, False, True),
        (2008, False, False),
    ],
)
def test_show_retrieve_best_ofs_details_by_year(
    year: int, inclusive: bool, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_best_ofs_details_by_year`.

    :param year: Four digit year to test retrieving show information
    :param inclusive: Include Repeat shows in the list of Best Of
        shows
    :param include_decimal_scores: Flag set to include decimal score
        columns and values

    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_best_ofs_details_by_year(
        year=year, inclusive=inclusive, include_decimal_scores=include_decimal_scores
    )

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "best_of" in shows[0], "'best_of' was not returned for the first list item"
    assert shows[0]["best_of"], "'best_of' value for the first list item is not valid"
    assert "panelists" in shows[0], (
        "'panelists' was not returned for the first list item"
    )


@pytest.mark.parametrize("year, month, day", [(2020, 4, 25)])
def test_show_retrieve_by_date(year: int, month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_date`.

    :param year: Four digit year to test retrieving a show's information
    :param month: One or two digit month to test retrieving a show's
        information
    :param day: One or two digit day to test retrieving a show's
        information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_date(year, month, day)

    assert info, f"Show for date {year:04d}-{month:02d}-{day:02d} not found"
    assert "date" in info, (
        f"'date' was not returned for show {year:04d}-{month:02d}-{day:02d}"
    )


@pytest.mark.parametrize("date", ["2018-10-27"])
def test_show_retrieve_by_date_string(date: str):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_date_string`.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving a show's information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_date_string(date)

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"


@pytest.mark.parametrize("show_id", [1162])
def test_show_retrieve_by_id(show_id: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_id`.

    :param show_id: Show ID to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_id(show_id)

    assert info, f"Show ID {show_id} not found"
    assert "date" in info, f"'date' was not returned for ID {show_id}"


@pytest.mark.parametrize("month, day", [(10, 28), (8, 19)])
def test_show_retrieve_by_month_day(month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_month_day`.

    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_month_day(month, day)

    assert shows, f"No shows could be retrieved for month {month:02d} and day {day:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list item "
        f"for month {month:02d} and day {day:02d}"
    )


@pytest.mark.parametrize("year", [2018])
def test_show_retrieve_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_year`.

    :param year: Four digit year to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_year(year)

    assert shows, f"No shows could be retrieved for year {year:04d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list item for year {year:04d}"
    )


@pytest.mark.parametrize("year, month", [(1998, 1), (2018, 10)])
def test_show_retrieve_by_year_month(year: int, month: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_year_month`.

    :param year: Four digit year to test retrieving show information
    :param month: One or two digit month to test retrieving show
        information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_year_month(year, month)

    assert shows, f"No shows could be retrieved for year/month {year:04d}-{month:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list "
        f"item for year/month {year:02d}-{month:04d}"
    )


@pytest.mark.parametrize(
    "year, inclusive", [(1998, True), (1998, False), (2010, True), (2010, False)]
)
def test_show_retrieve_counts_by_year(year: int, inclusive: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_counts_by_year`.

    :param year: Four digit year to test retrieving show counts
    """
    show = Show(connect_dict=get_connect_dict())
    counts = show.retrieve_counts_by_year(year=year, inclusive=inclusive)

    assert counts, f"No show counts were returned for year {year:04d}"
    assert "regular" in counts, f"No regular show count returned for year {year:04d}"
    assert counts["regular"] is not None, (
        f"Invalid regular show count for year {year:04d}"
    )
    assert "best_of" in counts, f"No Best Of show count returned for year {year:04d}"
    assert counts["best_of"] is not None, (
        f"Invalid Best Of show count for year {year:04d}"
    )
    assert "repeat" in counts, f"No repeat show count returned for year {year:04d}"
    assert counts["repeat"] is not None, (
        f"Invalid repeat show count for year {year:04d}"
    )
    assert "repeat_best_of" in counts, (
        f"No repeat Best Of show count returned for year {year:04d}"
    )
    assert counts["repeat_best_of"] is not None, (
        f"Invalid repeat Best Of show count for year {year:04d}"
    )
    assert "total" in counts, f"No total show count returned for year {year:04d}"
    assert counts["total"] is not None, (
        f"Incorrect total show count for year {year:04d}"
    )
    assert (
        counts["total"]
        == counts["regular"]
        + counts["best_of"]
        + counts["repeat"]
        + counts["repeat_best_of"]
    ), f"Total show count does not match actual total show count for year {year:04d}"


def test_show_retrieve_all_counts_by_year():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_counts_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    counts = show.retrieve_all_counts_by_year()

    assert counts, "No show counts were returned"
    assert 1998 in counts, "No show count information returned for year 1998"
    assert "regular" in counts[1998], "No regular show count returned for year 1998"
    assert counts[1998]["regular"] is not None, (
        "Invalid regular show count returned for 1998"
    )
    assert "best_of" in counts[1998], "No Best Of show count returned for year 1998"
    assert counts[1998]["best_of"] is not None, (
        "Invalid Best Of show count returned for 1998"
    )
    assert "repeat" in counts[1998], "No repeat show count returned for year 1998"
    assert counts[1998]["repeat"] is not None, (
        "Invalid repeat show count returned for 1998"
    )
    assert "repeat_best_of" in counts[1998], (
        "No repeat Best Of show count returned for year 1998"
    )
    assert counts[1998]["repeat_best_of"] is not None, (
        "Invalid repeat Best Of show count returned for 1998"
    )


@pytest.mark.parametrize(
    "year, month, day, include_decimal_scores",
    [(2020, 4, 25, True), (2020, 4, 25, False)],
)
def test_show_retrieve_details_by_date(
    year: int, month: int, day: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date`.

    :param year: Four digit year to test retrieving show details
    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date(
        year, month, day, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show for date {year:04d}-{month:02d}-{day:02d} not found"
    assert "date" in info, (
        f"'date' was not returned for show {year:04d}-{month:02d}-{day:02d}"
    )
    assert "host" in info, (
        f"'host' was not returned for show {year:04d}-{month:02d}-{day:02d}"
    )


@pytest.mark.parametrize(
    "date, include_decimal_scores", [("2018-10-27", True), ("2018-10-27", False)]
)
def test_show_retrieve_details_by_date_string(date: str, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date_string`.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date_string(
        date, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"
    assert "host" in info, f"'host' was not returned for show {date}"


@pytest.mark.parametrize("date", ["1999-02-13", "2018-10-27"])
def test_show_retrieve_details_by_date_string_decimal(date: str):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date_string` with decimal scores.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date_string(date, include_decimal_scores=True)

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"
    assert "host" in info, f"'host' was not returned for show {date}"


@pytest.mark.parametrize(
    "show_id, include_decimal_scores",
    [(1162, True), (1162, False), (1246, True), (1246, False)],
)
def test_show_retrieve_details_by_id(show_id: int, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_id`.

    :param show_id: Show ID to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_id(
        show_id, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show ID {show_id} not found"
    assert "date" in info, f"'date' was not returned for ID {show_id}"
    assert "host" in info, f"'host' was not returned for ID {show_id}"


@pytest.mark.parametrize(
    "month, day, include_decimal_scores",
    [(10, 28, True), (10, 28, False), (8, 19, True), (8, 19, False)],
)
def test_show_retrieve_details_by_month_day(
    month: int, day: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_month_day`.

    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_details_by_month_day(
        month, day, include_decimal_scores=include_decimal_scores
    )

    assert shows, f"No shows could be retrieved for month {month:02d} and day {day:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list item "
        f"for month {month:02d} and day {day:02d}"
    )


@pytest.mark.parametrize("year, include_decimal_scores", [(2021, True), (2021, False)])
def test_show_retrieve_details_by_year(year: int, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_year`.

    :param year: Four digit year to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_year(
        year, include_decimal_scores=include_decimal_scores
    )

    assert info, f"No shows could be retrieved for year {year:04d}"
    assert "date" in info[0], (
        f"'date' was not returned for first list item for year {year:04d}"
    )
    assert "host" in info[0], (
        f"'host' was not returned for first list item for year {year:04d}"
    )


@pytest.mark.parametrize(
    "year, month, include_decimal_scores", [(2020, 4, True), (2020, 4, False)]
)
def test_show_retrieve_details_by_year_month(
    year: int, month: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_year_month`.

    :param year: Four digit year to test retrieving show details
    :param month: One or two digit year to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_year_month(
        year, month, include_decimal_scores=include_decimal_scores
    )

    assert info, f"No shows could be retrieved for year/month {year:04d}-{month:02d}"
    assert "date" in info[0], (
        f"'date' was not returned for first list item "
        f"for year/month {year:04d}-{month:02d}"
    )
    assert "host" in info[0], (
        f"'host' was not returned for first list item "
        f"for year/month {year:04d}-{month:02d}"
    )


@pytest.mark.parametrize("year", [2018])
def test_show_retrieve_months_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_months_by_year`.

    :param year: Four digit year to test retrieving a list of months
    """
    show = Show(connect_dict=get_connect_dict())
    months = show.retrieve_months_by_year(year)

    assert months, f"No months could be retrieved for year {year:04d}"


def test_show_retrieve_recent():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_recent`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_recent()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_recent_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_recent_details`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_recent_details(include_decimal_scores=include_decimal_scores)

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


@pytest.mark.parametrize("year", [2008, 2012])
def test_show_retrieve_repeat_best_ofs_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_repeat_best_ofs_by_year`.

    :param year: Four digit year to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_repeat_best_ofs_by_year(year=year)

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "best_of" in shows[0], "'best_of' was not returned for the first list item"
    assert shows[0]["best_of"], "'best_of' value for the first list item is not valid"
    assert "repeat_show" in shows[0], (
        "'repeat_show' was not returned for the first list item"
    )
    assert shows[0]["repeat_show"], (
        "'repeat_show' value for the first list item is not valid"
    )


@pytest.mark.parametrize("year", [2008, 2012])
def test_show_retrieve_repeat_best_ofs_details_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_repeat_best_ofs_details_by_year`.

    :param year: Four digit year to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_repeat_best_ofs_details_by_year(year=year)

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "best_of" in shows[0], "'best_of' was not returned for the first list item"
    assert shows[0]["best_of"], "'best_of' value for the first list item is not valid"
    assert "repeat_show" in shows[0], (
        "'repeat_show' was not returned for the first list item"
    )
    assert shows[0]["repeat_show"], (
        "'repeat_show' value for the first list item is not valid"
    )
    assert "panelists" in shows[0], (
        "'panelists' was not returned for the first list item"
    )


@pytest.mark.parametrize(
    "year, inclusive",
    [
        (2000, True),
        (2000, False),
        (2010, True),
        (2010, False),
    ],
)
def test_show_retrieve_repeats_by_year(year: int, inclusive: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_best_ofs_by_year`.

    :param year: Four digit year to test retrieving show information
    :param inclusive: Include Best Of shows in the list of repeat
        shows
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_repeats_by_year(year=year, inclusive=inclusive)

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "repeat_show" in shows[0], (
        "'repeat_show' was not returned for the first list item"
    )
    assert shows[0]["repeat_show"], (
        "'repeat_show' value for the first list item is not valid"
    )


@pytest.mark.parametrize(
    "year, inclusive, include_decimal_scores",
    [
        (2000, True, True),
        (2000, True, False),
        (2000, False, True),
        (2000, False, False),
        (2010, True, True),
        (2010, True, False),
        (2010, False, True),
        (2010, False, False),
    ],
)
def test_show_retrieve_repeats_details_by_year(
    year: int, inclusive: bool, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_best_ofs_details_by_year`.

    :param year: Four digit year to test retrieving show information
    :param inclusive: Include Best Of shows in the list of repeat
        shows
    :param include_decimal_scores: Flag set to include decimal score
        columns and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_repeats_details_by_year(
        year=year, inclusive=inclusive, include_decimal_scores=include_decimal_scores
    )

    assert shows, f"Information for Best Of Shows for year {year:04d} not found"
    assert "id" in shows[0], "'id' was not returned for the first list item"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "repeat_show" in shows[0], (
        "'repeat_show' was not returned for the first list item"
    )
    assert shows[0]["repeat_show"], (
        "'repeat_show' value for the first list item is not valid"
    )
    assert "panelists" in shows[0], (
        "'panelists' was not returned for the first list item"
    )


@pytest.mark.parametrize("year, use_decimal_scores", [(2018, True), (2018, False)])
def test_show_retrieve_scores_by_year(year: int, use_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_scores_by_year`.

    :param year: Four digit year to test retrieving scores for a show
        year
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    scores = show.retrieve_scores_by_year(year, use_decimal_scores=use_decimal_scores)

    assert scores, f"No scores could be retrieved by year {year:04d}"
    assert isinstance(scores[0], tuple), "First list item is not a tuple"


def test_show_retrieve_years():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_years`."""
    show = Show(connect_dict=get_connect_dict())
    years = show.retrieve_years()

    assert years, "No years could be retrieved"
    assert isinstance(years[0], int), "First list item is not a number"


def test_show_retrieve_random_id() -> None:
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_id`."""
    show = Show(connect_dict=get_connect_dict())
    _id = show.retrieve_random_id()

    assert _id, "Returned random show ID is not valid"
    assert isinstance(_id, int), "Returned random show ID is not an integer"


@pytest.mark.parametrize("year", [1998, 2020])
def test_show_retrieve_random_id_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_id_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    _id = show.retrieve_random_id_by_year(year=year)

    assert _id, "Returned random show ID is not valid"
    assert isinstance(_id, int), "Returned random show ID is not an integer"

    _show = show.retrieve_by_id(show_id=_id)

    assert _show, f"Returned random show data for {_id} is not valid"
    assert str(year) in _show["date"], f"Show date for {_id} is not from {year}"


def test_show_retrieve_random_date() -> None:
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_date`."""
    show = Show(connect_dict=get_connect_dict())
    _date = show.retrieve_random_date()

    assert _date, "No random show date string returned"
    assert isinstance(_date, str), "Returned random show date string is not a string"


def test_show_retrieve_random_date_object() -> None:
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_date_object`."""
    show = Show(connect_dict=get_connect_dict())
    _date = show.retrieve_random_date_object()

    assert _date, "No random show date returned"
    assert isinstance(_date, datetime.date), (
        "Returned random show date is not a valid date object"
    )
    assert _date.year, "Random show date year is not valid"
    assert _date.month, "Random show date month is not valid"
    assert _date.day, "Random show date day is not valid"


@pytest.mark.parametrize("year", [1998, 2020])
def test_show_retrieve_random_date_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_date_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    _date = show.retrieve_random_date_by_year(year=year)

    assert _date, "Returned random show ID is not valid"
    assert isinstance(_date, str), "Returned random show ID is not an integer"
    assert str(year) in _date, f"Returned random show date is not from {year}"

    _show = show.retrieve_by_date_string(date_string=_date)

    assert _show, f"Returned random show data for {_date} is not valid"


@pytest.mark.parametrize("year", [1998, 2020])
def test_show_retrieve_random_date_object_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_date_object_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    _date = show.retrieve_random_date_object_by_year(year=year)

    assert _date, "No random show date returned"
    assert isinstance(_date, datetime.date), (
        "Returned random show date is not a valid date object"
    )
    assert _date.year, "Random show date year is not valid"
    assert _date.month, "Random show date month is not valid"
    assert _date.day, "Random show date day is not valid"


def test_show_retrieve_random() -> None:
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random`."""
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_random()

    assert info, "Random show not found"
    assert "date" in info, "'date' was not returned for a random show"


@pytest.mark.parametrize("year", [1998, 2020])
def test_show_retrieve_random_by_year(year: int) -> None:
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_random_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_random_by_year(year=year)

    assert info, "Random show not found"
    assert "date" in info, "'date' was not returned for a random show"
    assert str(year) in info["date"], f"Returned random show data is not from {year}"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_random_details(include_decimal_scores: bool) -> None:
    """Testing for :py:meth:`wwdtm.panelist.Show.retrieve_random_details`."""
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_random_details(include_decimal_scores=include_decimal_scores)

    assert info, "Random show not found"
    assert "date" in info, "'date' was not returned for a random show"
    assert "host" in info, "'host' was not returned for a random show"


@pytest.mark.parametrize(
    "year, include_decimal_scores",
    ([1998, True], [1998, False], [2020, True], [2020, False]),
)
def test_show_retrieve_random_details_by_year(
    year: int, include_decimal_scores: bool
) -> None:
    """Testing for :py:meth:`wwdtm.panelist.Show.retrieve_random_details_by_year`."""
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_random_details_by_year(
        year=year, include_decimal_scores=include_decimal_scores
    )

    assert info, "Random show not found"
    assert "date" in info, "'date' was not returned for a random show"
    assert str(year) in info["date"], f"Returned random show data is not from {year}"
    assert "host" in info, "'host' was not returned for a random show"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Panelist Scores Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.panelist.utility import PanelistUtility
from wwdtm.validation import valid_int_id


class PanelistScores:
    """Panelist score retrieval class.

    Contains methods used to retrieve panelist scores and calculate
    scoring statistics.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = PanelistUtility(database_connection=self.database_connection)

    def retrieve_scores_by_id(self, panelist_id: int) -> list[int]:
        """Retrieves a list of panelist scores, sorted by show date.

        :param panelist_id: Panelist ID
        :return: A list containing panelist decimal scores
        """
        if not valid_int_id(panelist_id):
            return []

        query = """
            SELECT pm.panelistscore AS score
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE panelistid = %s
            AND s.bestof = 0 and s.repeatshowid IS NULL
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (panelist_id,))
        result = cursor.fetchall()
        cursor.close()

        if not result:
            return []

        scores = []
        for appearance in result:
            if appearance["score"]:
                scores.append(appearance["score"])

        return scores

    def retrieve_scores_by_slug(self, panelist_slug: str) -> list[int]:
        """Retrieves a list of panelist scores, sorted by show date.

        :param panelist_slug: Panelist slug string
        :return: A list containing panelist scores
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return []

        return self.retrieve_scores_by_id(id_)

    def retrieve_scores_grouped_list_by_id(
        self, panelist_id: int
    ) -> dict[str, list[int]]:
        """Retrieves panelist grouped scores.

        :param panelist_id: Panelist ID
        :return: A dictionary containing two lists, one containing
            scores and one containing counts of those scores
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT MIN(pm.panelistscore) AS min,
            MAX(pm.panelistscore) AS max
            FROM ww_showpnlmap pm
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        result = cursor.fetchone()

        if not result:
            return {}

        min_score = result["min"]
        max_score = result["max"]

        scores = {}
        for score in range(min_score, max_score + 1):
            scores[score] = 0

        query = """
            SELECT pm.panelistscore AS score,
            COUNT(pm.panelistscore) AS score_count
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s
            AND s.bestof = 0 AND s.repeatshowid IS NULL
            AND pm.panelistscore IS NOT NULL
            GROUP BY pm.panelistscore
            ORDER BY pm.panelistscore ASC;
            """
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        for row in results:
            scores[row["score"]] = row["score_count"]

        return {
            "score": list(scores.keys()),
            "count": list(scores.values()),
        }

    def retrieve_scores_grouped_list_by_slug(
        self, panelist_slug: str
    ) -> dict[str, list[int]]:
        """Retrieves panelist grouped scores.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing two lists, one containing
            scores and one containing counts of those scores
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_scores_grouped_list_by_id(id_)

    def retrieve_scores_grouped_ordered_pair_by_id(
        self, panelist_id: int
    ) -> list[tuple[int, int]]:
        """Retrieves a list of panelist scores and counts as a tuple.

        :param panelist_id: Panelist ID
        :return: A list of tuples containing scores and score counts
        """
        if not valid_int_id(panelist_id):
            return []

        query = """
            SELECT MIN(pm.panelistscore) AS min,
            MAX(pm.panelistscore) AS max
            FROM ww_showpnlmap pm;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        result = cursor.fetchone()

        if not result:
            return []

        min_score = result["min"]
        max_score = result["max"]

        scores = {}
        for score in range(min_score, max_score + 1):
            scores[score] = 0

        query = """
            SELECT pm.panelistscore AS score,
            COUNT(pm.panelistscore) AS score_count
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s
            AND s.bestof = 0 AND s.repeatshowid IS NULL
            AND pm.panelistscore IS NOT NULL
            GROUP BY pm.panelistscore
            ORDER BY pm.panelistscore ASC;
            """
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        for row in results:
            scores[row["score"]] = row["score_count"]

        return list(scores.items())

    def retrieve_scores_grouped_ordered_pair_by_slug(
        self,
        panelist_slug: str,
    ) -> list[tuple[int, int]]:
        """Retrieves a list of panelist scores and counts as a tuple.

        :param panelist_slug: Panelist slug string
        :return: A list of tuples containing scores and score counts
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return []

        return self.retrieve_scores_grouped_ordered_pair_by_id(id_)

    def retrieve_scores_list_by_id(
        self,
        panelist_id: int,
    ) -> dict[str, list]:
        """Retrieves panelist appearances and scores as paired lists.

        :param panelist_id: Panelist ID
        :return: A dictionary containing a list show dates and a list
            of scores
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT s.showdate AS date, pm.panelistscore AS score
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s
            AND s.bestof = 0 AND s.repeatshowid IS NULL
            AND pm.panelistscore IS NOT NULL
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        show_list = []
        score_list = []
        for shows in results:
            show_list.append(shows["date"].isoformat())
            score_list.append(shows["score"])

        return {
            "shows": show_list,
            "scores": score_list,
        }

    def retrieve_scores_list_by_slug(
        self,
        panelist_slug: str,
    ) -> dict[str, list]:
        """Retrieves panelist appearances and scores as paired lists.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing a list show dates and a list
            of scores
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_scores_list_by_id(id_)

    def retrieve_scores_ordered_pair_by_id(
        self, panelist_id: int
    ) -> list[tuple[str, int]]:
        """Retrieves panelist appearances and scores as a list of tuples.

        :param panelist_id: Panelist ID
        :return: A list of tuples containing show dates and scores
        """
        if not valid_int_id(panelist_id):
            return []

        query = """
            SELECT s.showdate AS date, pm.panelistscore AS score
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s
            AND s.bestof = 0 AND s.repeatshowid IS NULL
            AND pm.panelistscore IS NOT NULL
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        scores = []
        for show in results:
            show_date = show["date"].isoformat()
            score = show["score"]
            scores.append((show_date, score))

        return scores

    def retrieve_scores_ordered_pair_by_slug(
        self,
        panelist_slug: str,
    ) -> list[tuple[str, int]]:
        """Retrieves panelist appearances and scores as a list of tuples.

        :param panelist_slug: Panelist slug string
        :return: A list of tuples containing show dates and scores
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return []

        return self.retrieve_scores_ordered_pair_by_id(id_)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.location.LocationUtility`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.location import LocationUtility


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("location_id", [95])
def test_location_utility_convert_id_to_slug(location_id: int):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.convert_id_to_slug`.

    :param location_id: Location ID to test converting into location
        slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(location_id)

    assert slug, f"Location slug for ID {location_id} was not found"


@pytest.mark.parametrize("location_id", [-1])
def test_location_utility_convert_invalid_id_to_slug(location_id: int):
    """Negative testing for :py:meth:`wwdtm.location.LocationUtility.convert_id_to_slug`.

    :param location_id: Location ID to test failing to convert into
        location slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(location_id)

    assert not slug, f"Location slug for ID {location_id} was found"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_utility_convert_slug_to_id(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.convert_slug_to_id`.

    :param location_slug: Location slug string to test converting into
        location ID
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_slug_to_id(location_slug)

    assert id_, f"Location ID for slug {location_slug} was not found"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-li"])
def test_location_utility_convert_invalid_slug_to_id(location_slug: str):
    """Negative testing for :py:meth:`wwdtm.location.LocationUtility.convert_slug_to_id`.

    :param location_slug: Location slug string to test failing to
        convert into location ID
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_slug_to_id(location_slug)

    assert not id_, f"Location ID for slug {location_slug} was found"


@pytest.mark.parametrize("location_id", [95])
def test_location_utility_id_exists(location_id: int):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.id_exists`.

    :param location_id: Location ID to test if a location exists
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(location_id)

    assert result, f"Location ID {location_id} does not exist"


@pytest.mark.parametrize("location_id", [-1])
def test_location_utility_id_not_exists(location_id: int):
    """Negative testing for :py:meth:`wwdtm.location.LocationUtility.id_exists`.

    :param location_id: Location ID to test if a location does not exist
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(location_id)

    assert not result, f"Location ID {location_id} exists"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_utility_slug_exists(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slug_exists`.

    :param location_slug: Location slug string to test if a location
        exists
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(location_slug)

    assert result, f"Location slug {location_slug} does not exist"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-li"])
def test_location_utility_slug_not_exists(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slug_exists` with venue name.

    :param location_slug: Location slug string to test if a location
        does not exists
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(location_slug)

    assert not result, f"Location slug {location_slug} exists"


@pytest.mark.parametrize("city", ["Chicago"])
def test_location_utility_slugify_location_city(city: str):
    """Negative testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location`.

    Testing with city name.

    :param city: City to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    with pytest.raises(ValueError) as exception_info:
        _ = utility.slugify_location(city=city)

    assert str(exception_info.value) == "Invalid location information provided"


@pytest.mark.parametrize("city, state", [("Chicago", "IL")])
def test_location_utility_slugify_location_city_state(city: str, state: str):
    """Negative testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location`.

    Testing with city and state names.

    :param city: City to include in the slug string
    :param state: State to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    with pytest.raises(ValueError) as exception_info:
        _ = utility.slugify_location(city=city, state=state)

    assert str(exception_info.value) == "Invalid location information provided"


@pytest.mark.parametrize(
    "location_id, venue, city, state", [(2, "Chase Auditorium", "Chicago", "IL")]
)
def test_location_utility_slugify_location_full(
    location_id: int, venue: str, city: str, state: str
):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location`.

    Testing with location ID, venue, city and state names.

    :param location_id: Location ID to include in the slug string
    :param venue: Venue name to include in the slug string
    :param city: City to include in the slug string
    :param state: State to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.slugify_location(
        location_id=location_id, venue=venue, city=city, state=state
    )

    assert slug, "Unable to convert into a slug string"
    assert isinstance(slug, str), "Value returned is not a string"


@pytest.mark.parametrize("location_id, venue", [(2, "Chase Auditorium")])
def test_location_utility_slugify_location_venue(location_id: int, venue: str):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location` with venue name.

    :param location_id: Location ID to include in the slug string
    :param venue: Venue name to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.slugify_location(location_id=location_id, venue=venue)

    assert slug, "Unable to convert into a slug string"
    assert isinstance(slug, str), "Value returned is not a string"


@pytest.mark.parametrize("venue, city, state", [("Chase Auditorium", "Chicago", "IL")])
def test_location_utility_slugify_location_venue_city_state(
    venue: str, city: str, state: str
):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location`.

    :param venue: Venue name to include in the slug string
    :param city: City to include in the slug string
    :param state: State to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.slugify_location(venue=venue, city=city, state=state)

    assert slug, "Unable to convert into a slug string"
    assert isinstance(slug, str), "Value returned is not a string"


@pytest.mark.parametrize("location_id", [2])
def test_location_utility_slugify_location_id(location_id: int):
    """Testing for :py:meth:`wwdtm.location.LocationUtility.slugify_location` with venue, city and state names.

    :param location_id: Location ID to include in the slug string
    """
    utility = LocationUtility(connect_dict=get_connect_dict())
    slug = utility.slugify_location(location_id=location_id)

    assert slug, "Unable to convert into a slug string"
    assert isinstance(slug, str), "Value returned is not a string"
